@@ -19,7 +19,9 @@
  *
  */
 
+const DBus = imports.dbus;
 const Lang = imports.lang;
+const Mainloop = imports.mainloop;
 const Gettext = imports.gettext;
 const _ = imports.gettext.gettext;
 
@@ -36,15 +38,14 @@ const ChangeMonitor = imports.changeMonitor;
 const Documents = imports.documents;
 const Error = imports.error;
 const Format = imports.format;
+const GDataMiner = imports.gDataMiner;
 const Global = imports.global;
 const Main = imports.main;
 const MainWindow = imports.mainWindow;
 const Manager = imports.manager;
 const Notifications = imports.notifications;
-const OffsetController = imports.offsetController;
 const Path = imports.path;
 const Query = imports.query;
-const Searchbar = imports.searchbar;
 const Selections = imports.selections;
 const Sources = imports.sources;
 const TrackerController = imports.trackerController;
@@ -52,6 +53,8 @@ const Tweener = imports.util.tweener;
 const Utils = imports.utils;
 const WindowMode = imports.windowMode;
 
+const MINER_REFRESH_TIMEOUT = 60; /* seconds */
+
 function Application() {
     this._init();
 }
@@ -137,6 +140,21 @@ Application.prototype = {
 	this.application.set_app_menu(menu);
     },
 
+    _refreshMinerNow: function() {
+        this._miner.RefreshDBRemote(DBus.CALL_FLAG_START, Lang.bind(this,
+            function(res, error) {
+                if (error) {
+                    log('Error updating the GData cache: ' + error.toString());
+                    return;
+                }
+
+                Mainloop.timeout_add_seconds(MINER_REFRESH_TIMEOUT,
+                                             Lang.bind(this, this._refreshMinerNow));
+            }));
+
+        return false;
+    },
+
     _onStartup: function() {
         String.prototype.format = Format.format;
 
@@ -145,9 +163,6 @@ Application.prototype = {
         Tweener.init();
 
         Global.application = this;
-        Global.offsetController = new OffsetController.OffsetController();
-        Global.searchController = new Searchbar.SearchController();
-        Global.errorHandler = new Error.ErrorHandler();
 
         // connect to tracker
         try {
@@ -164,27 +179,25 @@ Application.prototype = {
             return;
         }
 
-        Global.connectionQueue = new TrackerController.TrackerConnectionQueue();
-        Global.sourceManager = new Sources.SourceManager();
-        Global.searchCategoryManager = new Searchbar.SearchCategoryManager();
-        Global.searchMatchManager = new Searchbar.SearchMatchManager();
-        Global.searchTypeManager = new Searchbar.SearchTypeManager();
-        Global.queryBuilder = new Query.QueryBuilder();
+        Global.initSearch();
+
         Global.changeMonitor = new ChangeMonitor.TrackerChangeMonitor();
-        Global.collectionManager = new Manager.BaseManager();
         Global.documentManager = new Documents.DocumentManager();
+        Global.errorHandler = new Error.ErrorHandler();
         Global.trackerController = new TrackerController.TrackerController();
         Global.selectionController = new Selections.SelectionController();
         Global.modeController = new WindowMode.ModeController();
         Global.notificationManager = new Notifications.NotificationManager();
 
+        // startup a refresh of the gdocs cache
+        this._miner = new GDataMiner.GDataMiner();
+        this._refreshMinerNow();
+
         this._initMenus();
         this._mainWindow = new MainWindow.MainWindow(this.application);
     },
 
     _commandLine: function(app, commandLine) {
-        app.activate();
-
         let args = commandLine.get_arguments();
         if (args.length) {
             let urn = args[0]; // gjs eats argv[0]
@@ -197,12 +210,17 @@ Application.prototype = {
                     function(cursor) {
                         if (!cursor)
                             return;
-                        let doc = Global.documentManager.createDocumentFromCursor(cursor);
-                        Global.documentManager.addItem(doc);
+
+                        let doc = Global.documentManager.addDocumentFromCursor(cursor);
                         Global.documentManager.setActiveItem(doc);
                     }));
             }
+        } else {
+            Global.modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
         }
+
+        app.activate();
+
         return 0;
     }
 };
