@@ -38,6 +38,8 @@ const Utils = imports.utils;
 const Lang = imports.lang;
 const Signals = imports.signals;
 
+const _COLLECTION_PLACEHOLDER_ID = 'collection-placeholder';
+
 // fetch all the collections a given item is part of
 function FetchCollectionsJob(urn) {
     this._init(urn);
@@ -90,7 +92,8 @@ FetchCollectionsJob.prototype = {
     },
 
     _emitCallback: function() {
-        this._callback(this._collections);
+        if (this._callback)
+            this._callback(this._collections);
     }
 };
 
@@ -186,7 +189,8 @@ FetchCollectionStateForSelectionJob.prototype = {
             collectionState[collIdx] = state;
         }
 
-        this._callback(collectionState);
+        if (this._callback)
+            this._callback(collectionState);
     }
 };
 
@@ -212,7 +216,8 @@ UpdateMtimeJob.prototype = {
                     log(e);
                 }
 
-                this._callback();
+                if (this._callback)
+                    this._callback();
             }));
     }
 };
@@ -263,7 +268,9 @@ SetCollectionForSelectionJob.prototype = {
             let job = new UpdateMtimeJob(this._collectionUrn);
             job.run(Lang.bind(this,
                 function() {
-                    this._callback();
+
+                    if (this._callback)
+                        this._callback();
                 }));
         }
     }
@@ -303,7 +310,8 @@ CreateCollectionJob.prototype = {
                 if (key == 'res')
                     this._createdUrn = val;
 
-                this._callback(this._createdUrn);
+                if (this._callback)
+                    this._callback(this._createdUrn);
             }));
     }
 };
@@ -321,7 +329,7 @@ function OrganizeCollectionModel() {
 OrganizeCollectionModel.prototype = {
     _init: function() {
         this.model = Gd.create_organize_store();
-        this._placeholderPath = null;
+        this._placeholderRef = null;
 
         this._collAddedId =
             Global.collectionManager.connect('item-added',
@@ -335,18 +343,6 @@ OrganizeCollectionModel.prototype = {
         job.run(Lang.bind(this, this._onFetchCollectionStateForSelection));
     },
 
-    _clearPlaceholder: function() {
-        // remove the placeholder if it's here
-        if (this._placeholderPath) {
-            let placeholderIter = this.model.get_iter(this._placeholderPath)[1];
-
-            if (placeholderIter) {
-                this.model.remove(placeholderIter);
-                this._placeholderPath = null;
-            }
-        }
-    },
-
     _findCollectionIter: function(item) {
         let retval = null;
 
@@ -366,7 +362,7 @@ OrganizeCollectionModel.prototype = {
     },
 
     _onFetchCollectionStateForSelection: function(collectionState) {
-        this._clearPlaceholder();
+        this.removePlaceholder();
 
         for (idx in collectionState) {
             let item = Global.collectionManager.getItemById(idx);
@@ -405,9 +401,38 @@ OrganizeCollectionModel.prototype = {
         this._refreshState();
     },
 
-    setPlaceholder: function(path) {
-        this._clearPlaceholder();
-        this._placeholderPath = path;
+    addPlaceholder: function() {
+        this.removePlaceholder();
+
+        let iter = this.model.append();
+        Gd.organize_store_set(this.model, iter,
+                              _COLLECTION_PLACEHOLDER_ID, '', OrganizeCollectionState.ACTIVE);
+
+        let placeholderPath = this.model.get_path(iter);
+        if (placeholderPath != null)
+            this._placeholderRef = Gtk.TreeRowReference.new(this.model, placeholderPath);
+
+        return placeholderPath;
+    },
+
+    removePlaceholder: function() {
+        // remove the placeholder if it's here
+        if (this._placeholderRef) {
+            let placeholderPath = this._placeholderRef.get_path();
+            let placeholderIter = this.model.get_iter(placeholderPath)[1];
+
+            if (placeholderIter)
+                this.model.remove(placeholderIter);
+
+            this._placeholderRef = null;
+        }
+    },
+
+    forgetPlaceholder: function() {
+        let path = this._placeholderRef.get_path();
+        this._placeholderRef = null;
+
+        return path;
     },
 
     destroy: function() {
@@ -429,8 +454,6 @@ function OrganizeCollectionView() {
 
 OrganizeCollectionView.prototype = {
     _init: function() {
-        this._addCollectionPath = null;
-
         this._model = new OrganizeCollectionModel();
         this.widget = new Gtk.TreeView({ headers_visible: false,
                                          vexpand: true,
@@ -466,7 +489,6 @@ OrganizeCollectionView.prototype = {
 
         this._rendererText.connect('edited', Lang.bind(this, this._onTextEdited));
         this._rendererText.connect('editing-canceled', Lang.bind(this, this._onTextEditCanceled));
-        this._rendererText.connect('editing-started', Lang.bind(this, this._onTextEditStarted));
 
         this.widget.show();
     },
@@ -482,65 +504,56 @@ OrganizeCollectionView.prototype = {
         job.run(Lang.bind(this,
             function() {
                 this._model.refreshCollectionState();
-
-                // FIXME: we shouldn't be this, but tracker doesn't
-                // notify us for collection changes...
-                let coll = Global.collectionManager.getItemById(collUrn);
-                coll.refresh();
             }));
     },
 
-    _onTextEdited: function(cell, pathStr, newText) {
-        let path = Gtk.TreePath.new_from_string(pathStr);
-        let iter = this._model.model.get_iter(path)[1];
+    _onNewCollectionCreated: function(createdUrn) {
+        if (!createdUrn) {
+            this._model.removePlaceholder();
+            return;
+        }
 
+        let path = this._model.forgetPlaceholder();
+        if (!path)
+            return;
+
+        let iter = this._model.model.get_iter(path)[1];
+        this._model.model.set_value(iter, OrganizeModelColumns.ID, createdUrn);
+
+        let job = new SetCollectionForSelectionJob(createdUrn, true);
+        job.run(null);
+    },
+
+    _onTextEdited: function(cell, pathStr, newText) {
         cell.editable = false;
 
         if (!newText || newText == '') {
             // don't insert collections with empty names
-            this._model.model.remove(iter);
+            this._model.removePlaceholder();
             return;
-        } else {
-            // update the new name immediately
-            this._model.model.set_value(iter, OrganizeModelColumns.NAME, newText);
         }
 
+        // update the new name immediately
+        let path = Gtk.TreePath.new_from_string(pathStr);
+        let iter = this._model.model.get_iter(path)[1];
+        this._model.model.set_value(iter, OrganizeModelColumns.NAME, newText);
+
         // actually create the new collection
         let job = new CreateCollectionJob(newText);
-        job.run(Lang.bind(this, this._onCollectionCreated));
-    },
-
-    _onCollectionCreated: function(collUrn) {
-        // FIXME: we shouldn't be doing any of this, but tracker doesn't
-        // notify us for collection changes...
-
-        let job = new Documents.SingleItemJob(collUrn);
-        job.run(Query.QueryFlags.UNFILTERED, Lang.bind(this,
-            function(cursor) {
-                if (cursor)
-                    Global.documentManager.addDocumentFromCursor(cursor);
-            }));
+        job.run(Lang.bind(this, this._onNewCollectionCreated, path));
     },
 
     _onTextEditCanceled: function() {
-        if (this._addCollectionPath) {
-            let path = Gtk.TreePath.new_from_string(this._addCollectionPath);
-            let iter = this._model.model.get_iter(path)[1];
-
-            this._model.model.remove(iter);
-            this._addCollectionPath = null;
-        }
-    },
-
-    _onTextEditStarted: function(cell, editable, pathStr) {
-        this._addCollectionPath = pathStr;
+        this._model.removePlaceholder();
     },
 
     _checkCellFunc: function(col, cell, model, iter) {
         let state = model.get_value(iter, OrganizeModelColumns.STATE);
+        let id = model.get_value(iter, OrganizeModelColumns.ID);
 
         cell.active = (state & OrganizeCollectionState.ACTIVE);
         cell.inconsistent = (state & OrganizeCollectionState.INCONSISTENT);
+        cell.visible = (id != _COLLECTION_PLACEHOLDER_ID);
     },
 
     _detailCellFunc: function(col, cell, model, iter) {
@@ -557,12 +570,10 @@ OrganizeCollectionView.prototype = {
     },
 
     addCollection: function() {
-        let iter = this._model.model.append();
-        let path = this._model.model.get_path(iter);
+        let path = this._model.addPlaceholder();
 
-        Gd.organize_store_set(this._model.model, iter,
-                              'collection-placeholder', '', OrganizeCollectionState.NORMAL);
-        this._model.setPlaceholder(path);
+        if (!path)
+            return;
 
         this._rendererText.editable = true;
         this.widget.set_cursor_on_cell(path, this._viewCol, this._rendererText, true);
@@ -583,15 +594,21 @@ OrganizeCollectionDialog.prototype = {
                                        modal: true,
                                        destroy_with_parent: true,
                                        default_width: 400,
-                                       default_height: 200 });
-        this.widget.add_button('gtk-add', OrganizeCollectionDialogResponse.ADD);
+                                       default_height: 250 });
 
+        this.widget.add_button('gtk-add', OrganizeCollectionDialogResponse.ADD);
         this.widget.add_button('gtk-ok', Gtk.ResponseType.OK);
         this.widget.set_default_response(Gtk.ResponseType.OK);
 
         let contentArea = this.widget.get_content_area();
+        let sw = new Gtk.ScrolledWindow({ shadow_type: Gtk.ShadowType.IN,
+                                          margin_left: 5,
+                                          margin_right: 5,
+                                          margin_bottom: 3 });
         let collView = new OrganizeCollectionView();
-        contentArea.add(collView.widget);
+
+        sw.add(collView.widget);
+        contentArea.add(sw);
 
         this.widget.connect('response', Lang.bind(this,
             function(widget, response) {
@@ -599,7 +616,7 @@ OrganizeCollectionDialog.prototype = {
                     collView.addCollection();
             }));
 
-        this.widget.show();
+        this.widget.show_all();
     }
 };
 
@@ -611,6 +628,25 @@ SelectionController.prototype = {
     _init: function() {
         this._selection = [];
         this._selectionMode = false;
+
+        Global.documentManager.connect('item-removed',
+                                       Lang.bind(this, this._onDocumentRemoved));
+    },
+
+    _onDocumentRemoved: function(manager, item) {
+        let changed = false;
+        let filtered = this._selection.filter(Lang.bind(this,
+            function(value, index) {
+                if (item.id == value)
+                    changed = true;
+
+                return (item.id != value);
+            }));
+
+        if (changed) {
+            this._selection = filtered;
+            this.emit('selection-changed', this._selection);
+        }
     },
 
     setSelection: function(selection) {
@@ -684,7 +720,8 @@ SelectionToolbar.prototype = {
         this._leftBox.add(this._toolbarPrint);
         this._toolbarPrint.connect('clicked', Lang.bind(this, this._onToolbarPrint));
 
-        this._separator = new Gtk.SeparatorToolItem({ draw: false });
+        this._separator = new Gtk.SeparatorToolItem({ draw: false,
+                                                      visible: true });
         this._separator.set_expand(true);
         this.widget.insert(this._separator, -1);
 
@@ -794,9 +831,6 @@ SelectionToolbar.prototype = {
         else
             this._toolbarPrint.hide();
 
-        // if we're showing the favorite icon, also show the separator
-        this._separator.set_visible(showFavorite);
-
         if (canTrash)
             this._toolbarTrash.show_all();
         else
